/***************************************************************************
 * event.c
 *
 * Copyright (c) 2000 Stefan Hellkvist, Ericsson Radio Systems AB
 * Email: stefan@hellkvist.org
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *****************************************************************************/

/* Oct 2000 - ported to Pocket Viewer by Wittawat Yamwong <wittawat@web.de> */
/* 2000-2002 adapted for PVMerlin by Christian Ensel */


/*#define ALLOWDEBUGCODE /*ce enables some debug code; removed for PVMerlin V3.1 for code-size reasons */

#include "modeldef.h"

#include "md.h"

#include "point.h"
#include "event.h"
#include "pattern.h"
#include "wizard.h"

/*ce mode-indicators and buttons */
#include "modebut.h"

#include "mod_symb.h"

#include "params.h"


extern BANK bank;

static void paintPatternPart();
static void clearWriteArea();

/*ceV4 aus PV.c verlegt */
int pendown_count;
int pendown_x, pendown_y;


typedef struct {
    word object;
    bool ink;
    int  ch;
    int  xmid, x, y, width, height;
    Pattern FAR *current;
    t_pvm_mode pvmmode; /*ceV4 aus PV.c verlegt */
    word underlyingObj;
    int txtcursor_time_threshold; 
    bool allowExtmode;
    t_char_mode  char_mode;
    t_shift_mode shift_mode;
/*    word myObj; */
} PVMerlin;


static PVMerlin Merlin;

static BANK bank;
/*ce static MODE mode;*/
static int timer;

#if 0
/*ce statistical information*/
static int rawNrOfPoints;      /* number of points in original (raw) pattern */
static int rawStartSlope;      /* slope at beginning of raw pattern */
static int rawStartEndVectorX; /* x value of start-to-end-vector (x[last]-x[0]) of raw pattern */
static int rawStartEndVectorV; /* y value of ... (y[last]-y[0]) */
static int rawStartEndSlope;   /* slope of raw start-to-end-vector */

/*ce same statistical values for final (pre-processed) pattern*/
static int nrOfPoints;      /* number of points in final pattern (as used for elastic pattern matching); same as */
static int startSlope;      /* slope at beginning of pattern */
static int startEndVectorX; /* x value of start-to-end-vector (x[last]-x[0]) */
static int startEndVectorV; /* y value of ... (y[last]-y[0]) */
static int startEndSlope;   /* slope of start-to-end-vector */
#endif

/*ce4 backup/restore Merlin configuration*/
static PVMerlin merlinBak;

void mlBackupConfig()
{
    merlinBak = Merlin;
}

void mlRestoreConfig()
{
    Merlin = merlinBak;
}


static void
outputChar( unsigned char c )
{
    Merlin.ch = c;
}


/**
 * setMode
 * Sets the mode to either single or multiple mode
 **/
/*ce
void
setMode( MODE m )
{
    mode = m;
}
*/

void setCharMode( t_char_mode cm )
{
    Merlin.char_mode = cm;
}

void setShiftMode( t_shift_mode sm )
{
    Merlin.shift_mode = sm;
}

t_char_mode getCharMode()
{
    return (Merlin.shift_mode==SHIFTED_MODE) /* || shift_mode==CAPSLOCK_MODE ) */
            ? ALPHA_MODE  /* SHIFT_MODE always implies ALPHA_MODE */
            : Merlin.char_mode;
}

t_shift_mode getShiftMode()
{
    return Merlin.shift_mode;
}

unsigned char changeToNextMode( unsigned char c )
{    
    if ( c == UP_STROKE ) {
        if ( getShiftMode() == LOWERCASE_MODE )
            setShiftMode( SHIFTED_MODE );
        else if ( getShiftMode() == SHIFTED_MODE ) {
            setShiftMode( CAPSLOCK_MODE );
            setCharMode( ALPHA_MODE );
        }
        else /* if CAPSLOCK_MODE */
            setShiftMode( LOWERCASE_MODE );
        c=0;
    } 
    else if ( getShiftMode() == SHIFTED_MODE ) {  /* always fall back to lowercase-alpha from single-char shifted-mode */
        /*ce-new2 */
        setShiftMode( LOWERCASE_MODE );
        if ( c == '.' && Merlin.allowExtmode ) {
            setCharMode( ALPHA_EXT_MODE );
            c=0;
        }
        /*ce-new2-end */
        setCharMode( ALPHA_MODE );
        /*ce-new2 movedline setShiftMode( LOWERCASE_MODE ); */
    } 
    else {  /* the extended and numerical modes */
        t_char_mode cm = getCharMode();
        if ( cm == ALPHA_EXT_MODE ) {
            setCharMode( ALPHA_MODE );        
        }
        else if ( cm == NUMERAL_MODE ) {
            if ( c == '.'  && Merlin.allowExtmode ) {
                setCharMode( NUMERAL_EXT_MODE );
                c=0;
            } else if ( c == NUM_STROKE ) { 
                setCharMode( ALPHA_MODE );  
                c=0; /*ceV3.2 was there a little bug? */
            }      
            /*else leave NUMERAL_MODE */
        }
        else if ( cm == NUMERAL_EXT_MODE ) {
            setCharMode( NUMERAL_MODE );
        }
        else /* ALPHA_MODE */ {
            if ( c == '.'  && Merlin.allowExtmode ) {
                setCharMode( ALPHA_EXT_MODE );
                c=0;
            } else if ( c == NUM_STROKE ) {
                setCharMode( NUMERAL_MODE );        
                c=0; /*ceV3.2 was there a little bug? */
            }
            /*else leave ALPHA_MODE */           
        }
    }
    if (Merlin.allowExtmode)
        modeButtonsDisplayMode(); 
    return c;
}

/**
 * restoreBank
 * Restores the bank
 **/
void
restoreBank()
{
    /*setMode( SINGLE_STROKE );*/
    setBank( bank );
}


t_pvm_mode getEventMode()
{
    return Merlin.pvmmode;
}


/**
 * getMode
 * Returns the current mode
 **/
/*ce
MODE
getMode()
{
    return mode;
}
*/

/**
 * storeBank
 * Stores the bank in a safe place to beware of bank robbers perhaps :)
 **/
void
storeBank()
{
    bank = getBank();
}


#define MAXDOTSIZE 5

/**
 * timedOut
 * Called when a pattern is finished and we should analyze it.
 **/
static void
timedOut()
{
    unsigned char c;
    int xmin, xmax;
    int ymin, ymax;
    int isdot;

    /*ce changed whole logic */
    patternMinMax( Merlin.current, &xmin, &xmax , &ymin, &ymax );
    isdot =  (xmax-xmin <= MAXDOTSIZE) && (ymax-ymin <= MAXDOTSIZE);

    if ( isdot ) 
        c = '.'; /* '.' is a hard-wired extended character */
    else {
        Pattern FAR *np; /* pre-processed pattern */ 
        np = doallpreprocessing( Merlin.current );
        
#ifdef ALLOWDEBUGCODE
        if ( PARAM(PARAM_DEBUG) == P_DBG_PATTERN || PARAM(PARAM_DEBUG) == P_DBG_FULL ) {
            int i;
            for (i = 0; i < np->nPoints-1; i++) {
                LibGdsLine(  5 + (np->pv[i].x)  , 25 + (np->pv[i].y),
                             5 + (np->pv[i+1].x), 25 + (np->pv[i+1].y));
            }
        }
#endif
        
        if ( getCharMode() == NUMERAL_MODE )
            setBank( NUMERAL );
        else if ( getCharMode() == ALPHA_EXT_MODE )
            setBank( ALPHA_EXT );
        else if ( getCharMode() == NUMERAL_EXT_MODE )
            setBank( NUMERAL_EXT );
        else 
            setBank( ALPHA );

        c = recognize( np );

#ifdef ALLOWDEBUGCODE
        /*ce-new2 test recogn.speed */
        if ( PARAM(PARAM_DEBUG) == P_DBG_PATTERN || PARAM(PARAM_DEBUG) == P_DBG_FULL ) {
            byte h,m,s=100,s0=0;
            int sec,i;
            byte p = PARAM(PARAM_DEBUG);
            PARAM(PARAM_DEBUG) = 0;  /* turn debug off for measurement */
            LibGetTime2( &h, &m, &s0 );
            sec = (((int)h)*24 + m)*60;
            do {
                LibGetTime2( &h, &m, &s );
            } while (s==s0); /* wait for the next second to start (to make it a bit more accurate) */
            sec += s;
            for (i=0; i<50; i++)
                c = recognize( np );
            LibGetTime2( &h, &m, &s );
            sec = ((((int)h)*24 + m)*60+s) - sec; /* time diff */
            LibPutMsgDlg("sec: %i", sec );
            PARAM(PARAM_DEBUG) = p;
        }        
        /*ce-new2-end */
#endif
       
        if ( getCharMode() == ALPHA_MODE ) {
            #ifdef POLISH            
            c = recognizeMore( np, ALPHA_POLISH ); /* recognize extra polish characters */
            #endif
            if ( getShiftMode() == SHIFTED_MODE || getShiftMode() == CAPSLOCK_MODE ) {
                c = recognizeMore( np, ALPHA_CAPITAL );  /* also try extra capital patterns */
                c = internationalToupper(c);
            }
        }
        
        deletePattern( np );
    }
    
    c = changeToNextMode(c);
        
    /*ce not used here if ( mode == MULTIPLE_STROKE )
        restoreBank(); */

    if ( c == -1 ) 
        c=0;
    /* { */
    /*ce old stuff:
     if ( (c == 'i' || c == '1') && xmin > Merlin.xmid - 10 &&
         xmax < Merlin.xmid + 10 )
        c = 'I';        /* Stupid fix to write I */
        outputChar( c );
    /*}*/

    /*ce*/ 
    dropInputPattern();
        
}


void dropInputPattern()
{
    int i;
    Point FAR *pv;
    pv = Merlin.current->pv;
    for (i = 0; i < Merlin.current->nPoints-1; i++, pv++) {
        LibGdsLineCmp(pv[0].x, pv[0].y, pv[1].x, pv[1].y);
        LibGdsDotCmp(pv[0].x, pv[0].y);
    }
    deletePattern( Merlin.current );
    Merlin.current = newPattern( INITIAL_PATTERN_LENGTH, 'a' );
    LibPutDisp();
}

/**
 * penDown
 * Called when the user has put his foot, no, pen down
 **/
void
penDown( int x, int y )
{
    Point po;
    po.x = x;
    po.y = y;
    addPoint( Merlin.current, po );
    Merlin.ch = -1;
    /*if (Merlin.ink && Merlin.current->nPoints <= 1) {
        clearWriteArea();
        PUTDISP;
    }
    */
}


/**
 * penUp
 * Called when the pen is lifted from the pad
 **/
void
penUp( int x, int y )
{
#ifdef ALLOWDEBUGCODE
    if ( PARAM(PARAM_DEBUG) == P_DBG_PATTERN || PARAM(PARAM_DEBUG) == P_DBG_FULL ) {
        LibPutMsgDlg("up: %i,%i\rlast: %i,%i", x,y, Merlin.current->pv[Merlin.current->nPoints-1].x, Merlin.current->pv[Merlin.current->nPoints-1].y );
    }
#endif
    
    /* if ( mode == SINGLE_STROKE ) */
    timedOut();
    /* else
    timer = 1; */
}


/**
 * penDragged
 * Called when the user drags the pen over the pad
 **/
void
penDragged( int x, int y )
{
    Point po;
    int n;

    if (Merlin.current->nPoints >= MAX_PATTERN_LENGTH)
       return;   /*ce longer patterns are not allowed. Although pattern.c allows dynamic growth of 
                   patterns (which is used if INITIAL length is exceeded) we stop growing at some point, 
                   because it simply would not make much sense (and we will run out of memory anyway) */

    po.x = x;
    po.y = y;
    n = Merlin.current->nPoints;
    addPoint( Merlin.current, po );
    if (Merlin.ink && Merlin.current->nPoints > 1 && Merlin.current->nPoints > n) {
        paintPatternPart();
    }
}

static void clearWriteArea()
{
    LibClrBox(Merlin.x, Merlin.y, Merlin.width, Merlin.height);
    LibBox(Merlin.x, Merlin.y, Merlin.width, Merlin.height, 1);
    /*LibMeshLine(Merlin.xmid, Merlin.y + 1, 1, Merlin.height - 2);*/
}

static void paintPatternPart()
{
    Point FAR *pv;
    int xmin, xmax, ymin, ymax;
    pv = &Merlin.current->pv[Merlin.current->nPoints-2];
    LibGdsLineCmp(pv[0].x, pv[0].y, pv[1].x, pv[1].y);
    LibGdsDotCmp(pv[0].x, pv[0].y);
    if (pv[0].x < pv[1].x ) {
        xmin = pv[0].x;
        xmax = pv[1].x;
    } else {
        xmin = pv[1].x;
        xmax = pv[0].x;
    }
    if (pv[0].y < pv[1].y ) {
        ymin = pv[0].y;
        ymax = pv[1].y;
    } else {
        ymin = pv[1].y;
        ymax = pv[0].y;
    }    
    LibPutDispBox(xmin, ymin, xmax-xmin+1, ymax-ymin+1); 
}

/* returns -1 for still drawing pen stroke 
            0 for pen stroke enden, but only internal mode change etc. happend, no significant character was produced 
            * recognized character code otherwise */
int mlHandleEvent(TCHSTS *tsts)
{
    switch (tsts->act) {
    case ACT_MAKE:
        penDown(tsts->x, tsts->y);
        break;

    case ACT_BREAK_IN:
    case ACT_BREAK:
    case ACT_MOVE_OUT:
        penUp(tsts->x, tsts->y);
        /*LibTchInit(); ce: why the heck whas this line here? */
        break;

    case ACT_MOVE_IN: /* should never happen, but to my suspicion, it happes on 450X Models. !? */
        /*penDown(tsts->x, tsts->y);
        break; */
    case ACT_MOVE:
    case ACT_DOWN_IN:
        penDragged(tsts->x, tsts->y);
        break;
    }
    return Merlin.ch;
}

void mlPaint()
{
    /*int i;
    Point FAR *pv; */

    if (!Merlin.ink) 
        return;
    /*ce clearWriteArea();
    pv = Merlin.current->pv;
    for (i = 0; i < Merlin.current->nPoints-2; i++) {
    LibGdsLine(pv[0].x, pv[0].y, pv[1].x, pv[1].y);
        pv++;
    }
    */
}

static bool firstInit=TRUE;

void mlInit(TCHTBL FAR *tch, bool ink, word underlyingObj, bool allowExtmode) /*, word myObj)*/
{
    /*ce-kb initWizard(); */
    Merlin.object = tch->obj;
    Merlin.ink    = ink;
    Merlin.x      = tch->x1;
    Merlin.y      = tch->y1;
    Merlin.width  = tch->x2 - tch->x1 + 1;
    Merlin.height = tch->y2 - tch->y1 + 1;
    Merlin.xmid   = (tch->x1 + tch->x2) / 2;
    Merlin.allowExtmode = allowExtmode;
    Merlin.char_mode  = ALPHA_MODE;
    Merlin.shift_mode = LOWERCASE_MODE;

    Merlin.underlyingObj = underlyingObj; /*ceV4*/
    /*Merlin.myObj = myObj;*/

    if (firstInit) {
        initWizard();
        Merlin.current= newPattern( INITIAL_PATTERN_LENGTH, 'a' );
        Merlin.txtcursor_time_threshold = 4;
        firstInit = FALSE;
    }
    setBank( ALPHA );
    /* setMode( SINGLE_STROKE ); */
    pendown_count = 0;
    Merlin.pvmmode = PVM_IDLE_MODE; /* ce4 */
}


static int ML_PASSTIME_THRESHOLD[] = { 4, 2, 3, 7 };

void mlSetPassTimeThreshold(byte pval) {
    if (pval < 0 || pval > 3 )
        pval = 0;
    Merlin.txtcursor_time_threshold = ML_PASSTIME_THRESHOLD[pval];
}


/***ce handle events (high level state machine) ***/

/*#define TXTCURSOR_TIME_THRESHOLD 4 <- now a parameter */
#define TXTCURSOR_DIST_THRESHOLD 4  /* 2 was ok, but not good in a car ;-) */

int mlHandleOrPassEvent(TCHSTS *ptsts) {
    int ch;
    switch (Merlin.pvmmode) {
      register dx,dy;
      case PVM_IDLE_MODE:
          if ( ptsts->act != ACT_MAKE )
              return MERLIN_IGNORE; /* nothing happend that would break the idle state */
          Merlin.pvmmode = PVM_PENSTART_MODE;
          pendown_count = 0;
          pendown_x = ptsts->x;
          pendown_y = ptsts->y;
      case PVM_PENSTART_MODE:
          pendown_count++;
          dx = pendown_x - ptsts->x;
          dy = pendown_y - ptsts->y;
          if ( dx > TXTCURSOR_DIST_THRESHOLD || dx < -TXTCURSOR_DIST_THRESHOLD
               || dy > TXTCURSOR_DIST_THRESHOLD || dy < -TXTCURSOR_DIST_THRESHOLD) /* pen moved to early (and too far) -> user is writing a letter */
              Merlin.pvmmode = PVM_WRITE_MODE;
          else if (pendown_count > Merlin.txtcursor_time_threshold  /* pen did not move for some time -> user placing the cursor / pressing an underlying button etc. */
                   && ptsts->act != ACT_BREAK_IN && ptsts->act != ACT_BREAK ) /* and pen is not released right now */ {
              dropInputPattern();
              Merlin.pvmmode = PVM_TXTCURSOR_MODE;
              ptsts->obj = Merlin.underlyingObj;
              ptsts->act = ACT_MAKE;
              return MERLIN_PASSTHROUGH_START;
          }
          /* pen-start-mode already writes on the screen, so we
             continue to next case-statement */
      case PVM_WRITE_MODE:
          if ( ptsts->act == ACT_MOVE_OUT ) {
              dropInputPattern();
              Merlin.pvmmode = PVM_IDLE_MODE;
              return MERLIN_IGNORE;
          }
          ch = mlHandleEvent(ptsts);
          if ( ch != -1 ) {
              Merlin.pvmmode = PVM_IDLE_MODE;
              return ch;
          }
          /*else /*ceV3.1 faster loop cut-off to get next event */
          /*   continue;
          break;*/
          return MERLIN_PENSTROKE;
      case PVM_TXTCURSOR_MODE: /* is actually a pass-through-mode */
          ptsts->obj = Merlin.underlyingObj;
          if ( ptsts->act == ACT_BREAK_IN /* || tsts.act == ACT_MOVE_OUT */
               || ptsts->act == ACT_BREAK) {
              /* end selection or cursor positioning */
              Merlin.pvmmode = PVM_IDLE_MODE;
              return MERLIN_PASSTHROUGH_END;
          }
          return MERLIN_PASSTHROUGH;
    }
}

