/* permanent.h
   2001/07/29 Marcin Woli\'nski

   Permanent storage of PV applets' data.  
   NOTE: Data amount must not exceed 3KB.  This is not checked by this
   module!

   Based on CASIO's 16-puzzle example.  

*/

/* The module is used as follows:

   In application program a type is to be declared for a structure
   containing all data items that are to be stored between program
   invocations.  At the begining of program execution the structure is
   loaded from file if such exists or initialized to all zeroes
   otherwise.  Then data items can be examined/changed and at any time
   saved back to file.

   File for storing data is identified by a name of no more than 15
   chars (this is again not checked!).  This name will show up in the
   'Delete applications data' menu and right panel of PV Applications
   Manager.

*/

/* Initialization function: permanent_init()

   Parameters:
   fname :IN - name of data set
   data :OUT - pointer to pointer to data buffer
   dlen :IN - lenght of data buffer

   On succesful execution the function will allocate 'dlen' bytes and
   set '*data' to point to this allocated memory.  Then if a file of
   the specified name already exists, it will be read to the buffer.
   If not the buffer will be cleared to 0.  The result value signals
   whether data has been read from file (TRUE) or initialized to dummy
   (FALSE).  
*/

bool permanentInit( const char* fname, void **data, word dlen );

/* Save data: permanent_save()

   Call this whenever you want changes in the 'data' structure above
   to be reflected in the associated file.
*/

void permanentSave();

/* End */
